#include "potentials.hpp"
#include <cmath>

/**
 * Calculate the Lennard-Jones energy and force for the current particle pair
 * described by displacement vector r
 */
ForceAndEnergy LennardJones::calc_force_and_energy(Vec3<real> r) {
  real rmagsq = r.squared_norm2();
  if (rmagsq < this->m_rcutoffsq && rmagsq > 0.0) {
    real inv_rmag = 1 / std::sqrt(rmagsq);

    // Pre-Compute the terms (doing this saves on multiple devisions/pow
    // function call)
    real sigma_r = m_sigma / inv_rmag;
    real sigma_r5 = sigma_r * sigma_r * sigma_r * sigma_r * sigma_r;
    real sigma_r6 = sigma_r5 * sigma_r;
    real sigma_r11 = sigma_r5 * sigma_r5 * sigma_r;
    real sigma_r12 = sigma_r6 * sigma_r6;

    // Get the energy
    real energy = 4.0 * m_epsilon * (sigma_r12 - sigma_r6);

    // Get the force vector
    real force_mag = 4.0 * m_epsilon * (6.0 * sigma_r5 - 12.0 * sigma_r11);
    Vec3<real> force = r.scale(force_mag * inv_rmag);

    return {energy, force};

  } else {
    return ForceAndEnergy::zero();
  }
};
